package nckd.wnq01.wnq.scm.plugin.form;

import com.grapecity.documents.excel.Q;
import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * 2025-03-16 Tyx
 * 返利计算工具雷
 */
public class DiscountCalUtil {

    //客户上下级关系
    public String customerRelation = "nckd_customerrelation";
    //返利政策
    public String policy = "nckd_rebatepolicy1";
    //销售合同
    public String saleContract = "conm_salcontract";
    //销售订单
    public String saleOrder = "sm_salorder";
    //销售出库单
    public String saleIssue = "im_saloutbill";

    /**
     * 获取所有返还类型=计量计返还的客户及二级客户
     * @return
     */
    public DynamicObject[] getCustomer () {
        QFilter filter = new QFilter("nckd_returntype", QCP.equals, "jljfh");
        DynamicObject[] cols = BusinessDataServiceHelper.load(customerRelation, "nckd_customer,nckd_entryentity.nckd_ecustomer", new QFilter[]{filter});
        return cols;
    }

    /**
     * 按照销售部获取返利政策
     * @return
     */
    public DynamicObject[] getPolicy() {
        QFilter filter = new QFilter("nckd_rebatetype", QCP.equals, "1");
        DynamicObject[] cols = BusinessDataServiceHelper.load(policy, "nckd_saledept,nckd_completerate,entryentity.nckd_qtymin,entryentity.nckd_qtymax", new QFilter[]{filter});
        return cols;
    }

    /**
     * 获取返利政策dataSet
     * @return
     */
    public DataSet getPolicyDataSet() {
        QFilter filter = new QFilter("nckd_rebatetype", QCP.equals, "1");
        DataSet dataSet = QueryServiceHelper.queryDataSet("policy", policy, "entryentity.nckd_zbmc,nckd_saledept,nckd_completerate,entryentity.nckd_qtymin as qtymin,entryentity.nckd_qtymax as qtymax,entryentity.nckd_price as price", new QFilter[]{filter}, null);
        return dataSet;
    }

    /**
     * 获取销售合同，按客户和季度获取任务量
     * @return
     */
    public DataSet getContract(int year, int season) {
        String selectFields = "";
        switch(season) {
            case 1 :
                selectFields = "customer,nckd_qty1 + nckd_qty2 + nckd_qty3 as planQty";
                break;
            case 2 :
                selectFields = "customer,nckd_qty4 + nckd_qty5 + nckd_qty6 as planQty";
                break;
            case 3 :
                selectFields = "customer,nckd_qty7 + nckd_qty8 + nckd_qty9 as planQty";
                break;
            case 4 :
                selectFields = "customer,nckd_qty10 + nckd_qty11 + nckd_qty12 as planQty";
                break;
        }
        Calendar calendar = Calendar.getInstance();
        calendar.clear();
        calendar.set(Calendar.YEAR, year);
        Date endDate = calendar.getTime();
        calendar.roll(Calendar.DAY_OF_YEAR, -1);
        Date startDate = calendar.getTime();

        QFilter filter = new QFilter("biztime", QCP.large_equals, startDate);
        filter.and("biztime", QCP.less_equals, endDate);

        DataSet dataSet = QueryServiceHelper.queryDataSet("getContract", saleContract, selectFields, null, null);
        return dataSet;
    }

    /**
     * 获取销售订单销售数量，物料分类 like 袋装 45-03
     * @return
     */
    public DataSet getOrderQtyByCustomer (int year, int season) {
        QFilter filter1 = getFilterByYearAndSeason(year, season, saleOrder);
        filter1.and("billentry.material.masterid.group.longnumber", QCP.like, "45!45-03%");
        DataSet dataSet = QueryServiceHelper.queryDataSet("order", saleOrder, "customer,nckd_basedatafield1 as thht,billentry.material.masterid as material,billentry.qty as qty", new QFilter[]{filter1}, null);
        DataSet sumDataSet = dataSet.groupBy(new String[]{"customer","thht","material"}).sum("qty").finish();
        return sumDataSet;
    }

    /**
     * 获取销售出库数量，物料分类 like 散装 45-01
     * @return
     */
    public DataSet getIssueQtyByCustomer (int year, int season) {
        QFilter filter1 = getFilterByYearAndSeason(year, season, saleIssue);
        filter1.and("billentry.material.masterid.group.longnumber", QCP.like, "45!45-01%");
        DataSet dataSet = QueryServiceHelper.queryDataSet("issue", saleIssue, "customer,nckd_basedatafield as thht,billentry.material.masterid as material,billentry.qty as qty", new QFilter[]{filter1}, null);
        DataSet sumDataSet = dataSet.groupBy(new String[]{"customer","thht","material"}).sum("qty").finish();
        return sumDataSet;
    }

    public QFilter getFilterByYearAndSeason(int year, int season, String entityKey) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date startDate = null;
        Date endDate = null;
        try {
            if(season == 1) {
                startDate = sdf.parse(year +"-01-01");
                endDate = sdf.parse(year +"-03-31");
            }
            else if (season == 2) {
                startDate = sdf.parse(year +"-04-01");
                endDate = sdf.parse(year +"-06-30");
            }
            else if (season == 3) {
                startDate = sdf.parse(year +"-07-01");
                endDate = sdf.parse(year +"-09-30");
            }
            else if (season == 4) {
                startDate = sdf.parse(year +"-10-01");
                endDate = sdf.parse(year +"-12-31");
            }
            String bizdateKey = "";
            if(entityKey.equals(saleOrder)) {
                bizdateKey = "bizdate";
            }
            else {
                bizdateKey = "biztime";
            }

            QFilter filter = new QFilter(bizdateKey, QCP.less_equals, endDate);
            filter.and(bizdateKey, QCP.large_equals, startDate);
            return filter;

        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

}